#!/usr/bin/env python

from gimpfu import *

############################################################################
# include, then use: erro_msg to display errors in a box
#---------------------------------------------------------------------------
def erro_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(0) # box
    gimp.message ("ERROR: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
############################################################################
#

def MaxPixelColour (inImage, inDrawable) :
    ''' Applyes more color in a inDrawable, overlaying each pixel with max red, green, blue, yellow, cyan, magenta, white values.
    Note that this filter is fast, since it make use of tiles.
    '''
    
    pdb.gimp_context_push()
    # Indicates that the process has started.
    gimp.progress_init("Applying more colour in " + inDrawable.name + "...")

    # Set up an undo group, so the operation will be undone in one step.
    pdb.gimp_image_undo_group_start (inImage)

    # Get the inDrawable position.
    pos = 0;
    for i in range(len(inImage.layers)):
        if(inImage.layers[i] == inDrawable):
            pos = i
    
    # Create a new inDrawable to save the results (otherwise is not possible to undo the operation).
    #newLayer = gimp.Layer(inImage, inDrawable.name + " temp", inDrawable.width, inDrawable.height, inDrawable.type, inDrawable.opacity, inDrawable.mode)
    newLayer = gimp.Layer (inImage,  inDrawable.name+"_work", inDrawable.width, inDrawable.height, inDrawable.type, inDrawable.opacity, inDrawable.mode)
    inImage.add_layer (newLayer, pos)
    inDrawableName = inDrawable.name
    if '.' in inDrawableName:
        split = inDrawableName.split(".", 1)
        inDrawableName = split[0]
    
    # Clear the new inDrawable.
    pdb.gimp_edit_clear(newLayer)
    newLayer.mode = OVERLAY_MODE   
    newLayer.opacity = 50.00
    newLayer.flush()
    
    # Increase color contrast.
    try:
        # Calculate the number of tiles.
        tn = int(inDrawable.width / 64)
        if(inDrawable.width % 64 > 0):
            tn += 1
        tm = int(inDrawable.height / 64)
        if(inDrawable.height % 64 > 0):
            tm += 1
        
        # Iterate over the tiles.
        for i in range(tn):
            for j in range(tm):
                # Update the progress bar.
                gimp.progress_update (float(i*tm + j) / float(tn*tm))
        
                # Get the tiles.
                srcTile = inDrawable.get_tile(False, j, i)
                dstTile = newLayer.get_tile(False, j, i)
        
                # Iterate over the pixels of each tile.
                for x in range(srcTile.ewidth):
                    for y in range(srcTile.eheight):
                        # Get the pixel and calculate his R,G,B values.
                        pixel = srcTile[x,y]
                        red = ord(pixel[0])
                        green = ord(pixel[1])
                        blue = ord(pixel[2])
                        newred = red
                        newgreen = green
                        newblue = blue
                        
                        Red=red/10
                        Green=green/10
                        Blue=blue/10
                        Yellow=(Red+Green)/2
                        Magenta=(Red+Blue)/2
                        Cyan=(Green+Blue)/2
                        Total=(Red+Green+Blue)/3
                        
                        if (Red>Green and Red>Blue):   # red 
                            newred = 255
                            nregreen = 0
                            newblue = 0
                        if (Green>Red and Green>Blue):   # green
                            newgreen = 255
                            newred = 0
                            newblue = 0
                        if (Blue>Red and Blue>Green):   # blue
                            newblue = 255
                            newred = 0
                            nregreen = 0
 
                        if (Red==Green and Red>Blue):   # yellow 
                            newred = 255
                            newgreen = 255
                            newblue = 0
                        if (Blue==Green and Red<Blue):   # cyan 
                            newred =0
                            newgreen = 255
                            newblue = 255
                        if (Blue==Red and Green<Blue):   # magenta 
                            newred = 255
                            newgreen = 0
                            newblue = 255
                            
                        if (Red==Green and Green==Blue):   # white
                            newred = 255
                            newgreen = 255
                            newblue = 255
                            
                        res = chr(newred) + chr(newgreen) + chr(newblue)
                        
                        # If the image has an alpha channel (or any other channel) copy his values.
                        if (len(pixel) > 3):
                            for k in range(len(pixel)-3):
                                res += pixel[k+3]
                                
                        # Save the value in the result inDrawable.
                        dstTile[x,y] = res
        
        # Update the new inDrawable.
        newLayer.flush()
        newLayer.merge_shadow(True)
        newLayer.update(0, 0, newLayer.width, newLayer.height)
        
        # 
        x_blur = newLayer.width/50
        y_blur = newLayer.height/50
        pdb.plug_in_gauss_iir2 (inImage, newLayer, x_blur, y_blur)
   
        # Change the name of the new inDrawable (two inDrawables can not have the same name).
        newLayer.name = inDrawableName+"-cc"
        
        # increase the sharpness of the base image
        pdb.plug_in_unsharp_mask (inImage, inDrawable, 7.0, 0.7, 0)
     
        newLayer.name = inDrawableName+", with max R,G,B,Y,C,M,W values"
        
    except Exception as err:
        erro_msg ("Unexpected error: " + str(err))
    
    # Close the undo group.
    pdb.gimp_image_undo_group_end (inImage)
    
    # End progress.
    pdb.gimp_progress_end()
    pdb.gimp_context_pop()
    
    return (newLayer)

register(
    "MaxPixelColour",
    "MaxPixelColour",
    "Create a new overlaying layer where each source pixel is set to the max of the 7 colours R, G, B, Y, C, M, W.",
    "JFM-DN",
    "Open source (BSD 3-clause license)",
    "2017",
    "MaxPixelColour",
    "RGB, RGB*",
    [
      (PF_IMAGE, "image", "Input image", None),
      (PF_DRAWABLE, "drawable", "Input drawable", None),

    ],
    [
      (PF_DRAWABLE, "outdrawable", "Output drawable", None),
    ],
    MaxPixelColour,
    menu="<Image>/Diego/Ancillary/"
    )
main()
